<?php

namespace Drupal\site_guardian\Services;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\Core\Utility\ProjectInfo;
use Drupal\update\ProjectCoreCompatibility;
use Drupal\update\UpdateFetcherInterface;

class SiteGuardianService {

  protected $settings;

  public function __construct(ConfigFactoryInterface $config) {
    $this->settings = $config->get('site_guardian.settings');
  }

  public function getKey() {
    return $this->settings->get('site_guardian_key');
  }

  public function isActivated() {
    return boolval($this->settings->get('site_guardian_activated'));
  }

  public function getSiteNotes() {
    return $this->settings->get('site_guardian_notes');
  }

  /**
   * Checks for the activation status of this module, and for the Site Guardian Key to make sure the API cannot be called without it.
   */
  public function checkActivationAndSiteGuardianKey($site_guardian_key) {

    $savedKey = $this->settings->get('site_guardian_key');

    if (!$this->isActivated()) {

      \Drupal::logger('site_guardian')->warning("Site Guardian request received but Site Guardian API has not been activated.");

      return "Site Guardian has not been activated for this site. Please contact a site administrator.";
    }

    if ($savedKey && (empty($site_guardian_key) || $site_guardian_key != $savedKey)) {

      \Drupal::logger('site_guardian')->warning("Site Guardian request received but Site Guardian API access is denied.");

      return "Access denied";
    }

    return 'Checks passed';
  }

  public function accessAllowed($site_guardian_key) {
    
    if ($this->checkActivationAndSiteGuardianKey($site_guardian_key) === 'Checks passed') {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  public function getEnabledModulesAndUpdates() {

    $projects = [];
    
    $moduleHandler = \Drupal::moduleHandler();

    $moduleHandler->loadInclude('update', 'inc', 'update.compare');

    // First, get a list of projects. This part borrows code from get_projects()
    // in UpdateManager.php, before adjusting to include modules created using the
    // Features module, as well as custom modules/themes.
    $modules = \Drupal::service('extension.list.module')->reset()->getList();
    $themes = \Drupal::service('theme_handler')->rebuildThemeData();

    foreach ($modules as $module_name => $module_data) {
      if (preg_match('(modules/features)', $module_data->subpath)) {
        $module_data->info['project'] = $module_name . '_created_by_features_module';
      }
      if ($module_data->origin != 'core' && $module_data->status == '1' && !isset($module_data->info['project'])) {
        $module_data->info['project'] = $module_name;
      }
    }

    foreach ($themes as $theme_name => $theme_data) {
      if (preg_match('(themes/features)', $theme_data->subpath)) {
        $theme_data->info['project'] = $theme_name . '_created_by_features_module';
      }
      if ($theme_data->origin != 'core' && $theme_data->status == '1' && !isset($theme_data->info['project'])) {
        $theme_data->info['project'] = $theme_name;
      }
    }

    $project_info = new ProjectInfo();

    $project_info->processInfoList($projects, $modules, 'module', TRUE);
    $project_info->processInfoList($projects, $themes, 'theme', TRUE);

    $moduleHandler->alter('update_projects', $projects);
    // End of code borrowed from get_projects().

    // Now try to get available update data. This marks the start of amended
    // code borrowed from update_get_available in update.module.
    $needs_refresh = FALSE;
    $available = \Drupal::keyValueExpirable('update_available_releases')->getAll();

    foreach ($projects as $key => $project) {

      if (empty($available[$key])) {
        \Drupal::service('update.processor')->createFetchTask($project);
        $needs_refresh = TRUE;
        continue;
      }
      if ($project['info']['_info_file_ctime'] > $available[$key]['last_fetch']) {
        $available[$key]['fetch_status'] = UpdateFetcherInterface::FETCH_PENDING;
      }
      if (empty($available[$key]['releases']) && !$available[$key]['last_fetch']) {
        $available[$key]['fetch_status'] = UpdateFetcherInterface::FETCH_PENDING;
      }
      if (!empty($available[$key]['fetch_status']) && $available[$key]['fetch_status'] == UpdateFetcherInterface::FETCH_PENDING) {
        \Drupal::service('update.processor')->createFetchTask($project);
        $needs_refresh = TRUE;
      }
    }

    if ($needs_refresh) {
      update_fetch_data();
      $available = \Drupal::keyValueExpirable('update_available_releases')->getAll();
    }
    // End of borrowing from update_get_available.

    // Next, compare list of projects with available update data. This section
    // draws from update_calculate_project_data in update.compare.inc.
    update_process_project_info($projects);

    if (isset($projects['drupal']) && !empty($available['drupal'])) {
      update_calculate_project_update_status($projects['drupal'], $available['drupal']);
      if (isset($available['drupal']['releases'])) {
        $project_core_compatibility = new ProjectCoreCompatibility($projects['drupal'], $available['drupal']['releases']);
      }
    }

    foreach ($projects as $project => $project_info) {

      if (isset($available[$project])) {
        if ($project === 'drupal') {
          continue;
        }
        update_calculate_project_update_status($projects[$project], $available[$project]);
        if (isset($project_core_compatibility)) {
          $project_core_compatibility->setReleaseMessage($projects[$project]);
        }
      }
      else {
        $projects[$project]['status'] = UpdateFetcherInterface::UNKNOWN;
        $projects[$project]['reason'] = t('No available releases found');
      }
    }

    $moduleHandler->alter('update_status', $projects);
    // End of code adapted from update_calculate_project_data.

    // Finally, add a datestamp for when these checks were last performed, if ever.
    $projects['last_checked'] = \Drupal::state()->get('update.last_check') ?: 0;

    return $projects;
  }
}